@@ -1,5 +1,3 @@
-context("reconnect")
-
 roundtrip = function(x) {
   path = tempfile()
   on.exit(file.remove(path))
@@ -7,11 +5,12 @@ roundtrip = function(x) {
   readRDS(path)
 }
 
-test_that("expectations", {
-  path = tempfile("db_", fileext = "sqlite")
-  b = as_sqlite_backend(iris, path = path)
+test_that("expectations + dplyr", {
+  skip_if_not_installed("dplyr")
+  skip_if_not_installed("dbplyr")
+
+  b = as_sqlite_backend(iris, path = tempfile())
   on.exit(disconnect(b))
-  b$connector = sqlite_reconnector(path)
 
   b = roundtrip(b)
   expect_false(DBI::dbIsValid(private(b)$.data$src$con))
@@ -20,35 +19,24 @@ test_that("expectations", {
   expect_iris_backend(b)
 })
 
-test_that("resampling", {
-  future::plan("multisession")
-  path = tempfile("db_", fileext = "sqlite")
-  b = as_sqlite_backend(iris, path = path)
-  on.exit(disconnect(b))
-  b$connector = sqlite_reconnector(path)
 
-  task = mlr3::TaskClassif$new("iris-sqlite", b, target = "Species")
-  learner = mlr3::lrn("classif.featureless")
-  resampling = mlr3::rsmp("holdout")
+test_that("expectations + duckdb", {
+  skip_if_not_installed("duckdb")
 
-  rr = mlr3::resample(task, learner, resampling)
-  expect_resample_result(rr)
-  expect_data_table(rr$errors, nrows = 0L)
-
-  skip_if_not_installed("future")
-  skip_if_not_installed("future.apply")
-  skip_if_not_installed("future.callr")
-  requireNamespace("future.callr")
-  with_future(future.callr::callr, {
-    rr = mlr3::resample(task, learner, resampling)
-  })
-  expect_resample_result(rr)
-  expect_data_table(rr$errors, nrows = 0L)
+  b = as_duckdb_backend(iris, path = tempfile())
+  b = roundtrip(b)
+  expect_false(b$valid)
+  expect_backend(b)
+  expect_iris_backend(b)
+  disconnect(b)
 })
 
 test_that("filtered tbl", {
-  path = tempfile("db_", fileext = "sqlite")
-  b = as_sqlite_backend(cbind(iris, data.frame(row_id = 1:150)), path = path)
+  skip_if_not_installed("dplyr")
+  skip_if_not_installed("dbplyr")
+
+  b = as_sqlite_backend(cbind(iris, data.frame(row_id = 1:150)), primary_key = "row_id", path = tempfile())
+  path = extract_db_dir(b)
   on.exit(disconnect(b))
 
   keep = c("row_id", "Sepal.Length", "Petal.Length", "Species")
@@ -56,7 +44,7 @@ test_that("filtered tbl", {
   tbl = dplyr::tbl(con, "data")
   tbl = dplyr::select_at(tbl, keep)
   tbl = dplyr::filter(tbl, Species == "setosa")
-  expect_data_frame(collect(tbl), nrows = 50, ncols = 4)
+  expect_data_frame(dplyr::collect(tbl), nrows = 50, ncols = 4)
 
   b = DataBackendDplyr$new(tbl, "row_id")
   expect_equal(b$ncol, 4)
@@ -75,4 +63,6 @@ test_that("filtered tbl", {
   expect_equal(b$nrow, 50)
   expect_equal(b$ncol, 4)
   expect_set_equal(b$colnames, keep)
+
+  disconnect(b)
 })
